@@ -4,7 +4,7 @@
 /*
  * $Id: pam_env.c,v 1.3 1999/11/08 05:46:53 morgan Exp $
  * 
- * Written by Tollef Fog Heen <tollef@add.no> 2001-02-03
+ * Written by Tollef Fog Heen <tfheen@err.no> 2001-02-03
  *
  * Based on pam_env
  * Written by Dave Kinchlea <kinch@kinch.ark.com> 1997/01/31
@@ -48,10 +48,12 @@ static int make_tmp_directory() {
   /* Check whether the mkdir succeeded */
   if (ret == -1 && errno != EEXIST) {
     _log_err(LOG_NOTICE, "mkdir %s failed: %s\n", tmpdir, strerror(errno));
+    free(tmpdir);
     return 1;
   } else if (ret != -1) {
     if (chown(tmpdir, 0, 0) == -1) {
       _log_err(LOG_NOTICE, "chown 0:0 %s failed: %s\n", tmpdir, strerror(errno));
+      free(tmpdir);
       return 1;
     }
   }
@@ -70,18 +72,22 @@ static int make_tmp_directory() {
     _log_err(LOG_ERR, "%s is owned by uid %d instead of root " 
 	    "(uid 0). Failed to create safe $TMPDIR\n", tmpdir, 
              statbuf.st_uid);
+    free(tmpdir);
     return 1;
   } else if (!S_ISDIR(statbuf.st_mode)) {
     _log_err(LOG_NOTICE, "%s is not a directory. Failed to create safe "
              "$TMPDIR\n", tmpdir);
+    free(tmpdir);
     return 1;
   } else if ((statbuf.st_mode & S_IWGRP) || (statbuf.st_mode & S_IWOTH)) {
     _log_err(LOG_NOTICE, "%s is group or world writable. "
 	     "Failed to create safe $TMPDIR\n", tmpdir);
+    free(tmpdir);
     return 1;
   } else if (!(statbuf.st_mode & S_IXOTH)) {
     _log_err(LOG_NOTICE,"%s is not world searchable. "
 	     "Failed to create safe $TMPDIR\n", tmpdir); 
+    free(tmpdir);
     return 1;
   }
 
