@@ -4,7 +4,7 @@
 /*
  * $Id: pam_env.c,v 1.3 1999/11/08 05:46:53 morgan Exp $
  * 
- * Written by Tollef Fog Heen <tollef@add.no> 2001-02-03
+ * Written by Tollef Fog Heen <tfheen@err.no> 2001-02-03
  *
  * Based on pam_env
  * Written by Dave Kinchlea <kinch@kinch.ark.com> 1997/01/31
@@ -44,27 +44,24 @@
 #include <security/pam_modules.h>
 #include <security/_pam_macros.h>
 
-#define PAM_TMPDIR_HELPER "/sbin/pam-tmpdir-helper"
+#define PAM_TMPDIR_HELPER "/usr/libexec/pam-tmpdir/pam-tmpdir-helper"
 
 static int set_environment(pam_handle_t *pamh);
 static int make_tmp_directory(pam_handle_t *pamh);
 
-uid_t get_user_id(pam_handle_t *pamh) {
-  char *username = malloc(25);
+int get_user_id(pam_handle_t *pamh, uid_t *uid) {
+  const void *username;
   struct passwd* user_entry;
-  
-  pam_get_item(pamh,PAM_USER,(void*)&username);
-  user_entry = getpwnam(username);
-  return user_entry->pw_uid;
-}
+  int r;
 
-gid_t get_group_id(pam_handle_t *pamh) {
-  char *username = malloc(25);
-  struct passwd* user_entry;
-  
-  pam_get_item(pamh,PAM_USER,(void*)&username);
+  r = pam_get_item(pamh, PAM_USER, &username);
+  if (r != PAM_SUCCESS) {
+    _log_err(LOG_WARNING, "could not get PAM user");
+    return -1;
+  }
   user_entry = getpwnam(username);
-  return user_entry->pw_gid;
+  *uid = user_entry->pw_uid;
+  return 0;
 }
 
 /* argument parsing */
@@ -78,12 +75,19 @@ static int set_environment(pam_handle_t *pamh) {
   char *buf;
   char *tmpdir = get_tmp_dir();
   char *envs[] = { "TMP", "TMPDIR", "TEMP", "TEMPDIR", NULL };
+  uid_t u;
+  if (get_user_id(pamh, &u) != 0) {
+    free(tmpdir);
+    return -1;
+  }
+
   for (int i = 0; envs[i]; i++) {
-    asprintf(&buf, "%s=%s/%d", envs[i], tmpdir, get_user_id(pamh));
+    asprintf(&buf, "%s=%s/%d", envs[i], tmpdir, u);
     pam_putenv(pamh, buf);
     free(buf);
   }
 
+  free(tmpdir);
   return 0;
 }
 
@@ -110,9 +114,17 @@ static int make_tmp_directory(pam_handle_t *pamh) {
   pid = fork();
   if (!pid) {
     /* child */
-    if (geteuid() == 0)
-      setuid(get_user_id(pamh));
+    if (geteuid() == 0) {
+      uid_t u;
+      if (get_user_id(pamh, &u) != 0) {
+	exit(1);
+      }
+      if (setuid(u) != 0) {
+	exit(1);
+      }
+    }
     execl(PAM_TMPDIR_HELPER,PAM_TMPDIR_HELPER,NULL);
+    exit(1);
   }
 
   ret = waitpid(pid, &status, 0);
