@@ -1,6 +1,6 @@
 
 #ifndef _PAM_TMPDIR_UTIL_H_
-#define _PAM_TMPDIR_UTIL_H_ 
+#define _PAM_TMPDIR_UTIL_H_
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -12,7 +12,9 @@
 #include <string.h>
 #include <stdarg.h>
 #include <libgen.h>
+#include <pwd.h>
 
+#include <security/pam_modules.h>
 
 void *xrealloc(void *ptr, size_t size);
 void *xmalloc(size_t size);
@@ -21,5 +23,6 @@ int check_dir_ok(char *path);
 int check_path(const char *path);
 char *get_tmp_dir();
 void _log_err(int err, const char *format, ...);
+const char *get_homedir(pam_handle_t *pamh);
 
 #endif
